﻿using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Diagnostics;
using System.Windows.Forms;
using AddinExpress.MSO;

namespace OutlookItemEvents
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("7864D451-AF01-4509-9676-D5350991CC7E"), ProgId("OutlookItemEvents.AddinModule")]
    public class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        public AddinModule()
        {
            Application.EnableVisualStyles();
            InitializeComponent();
            // Please add any initialization code to the AddinInitialize event handler
        }

        private ADXOutlookAppEvents adxOutlookEvents;
        private OutlookItemEventsClass1 itemEvents;

        #region Component Designer generated code
        /// <summary>
        /// Required by designer
        /// </summary>
        private System.ComponentModel.IContainer components;

        /// <summary>
        /// Required by designer support - do not modify
        /// the following method
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.adxOutlookEvents = new AddinExpress.MSO.ADXOutlookAppEvents(this.components);
            // 
            // adxOutlookEvents
            // 
            this.adxOutlookEvents.InspectorActivate += new AddinExpress.MSO.ADXOlInspector_EventHandler(this.adxOutlookEvents_InspectorActivate);
            this.adxOutlookEvents.InspectorClose += new AddinExpress.MSO.ADXOlInspector_EventHandler(this.adxOutlookEvents_InspectorClose);
            // 
            // AddinModule
            // 
            this.AddinName = "OutlookItemEvents";
            this.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook;
            this.AddinBeginShutdown += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinBeginShutdown);
            this.AddinStartupComplete += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinStartupComplete);

        }
        #endregion

        #region Add-in Express automatic code

        // Required by Add-in Express - do not modify
        // the methods within this region

        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }

        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }

        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }

        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

        public static new AddinModule CurrentInstance
        {
            get
            {
                return AddinExpress.MSO.ADXAddinModule.CurrentInstance as AddinModule;
            }
        }

        public Outlook._Application OutlookApp
        {
            get
            {
                return (HostApplication as Outlook._Application);
            }
        }

        private void AddinModule_AddinStartupComplete(object sender, EventArgs e)
        {
            Debug.Print("The AddinStartupComplete event has occured. The add-in has been loaded by Outlook " + this.HostMajorVersion.ToString());

            // This creates an item of the class that handles the events of an Outlook item
            itemEvents = new OutlookItemEventsClass1(this);
        }

        private void AddinModule_AddinBeginShutdown(object sender, EventArgs e)
        {
            // in Outlook 2010, this event may not occur, see the blog 'Outlook 2010 Fast Shutdown feature'
            // at http://www.add-in-express.com/creating-addins-blog/2010/05/04/outlook2010-fast-shutdown/
            Debug.Print("The AddinBeginShutdown event has occured.");
            if (itemEvents.IsConnected)
            {
                Debug.Print("Disconnecting from the previously connected item");
                itemEvents.RemoveConnection();
            }
            itemEvents = null;
        }

        private void adxOutlookEvents_InspectorActivate(object sender, object inspector, string folderName)
        {
            object itemObj = (inspector as Outlook._Inspector).CurrentItem;
            // Now itemObj contains a COM object holding the Outlook item shown in the active inspector.
            // Creating a COM object implies releasing it as soon as you don't need it! See how we deal with this.

            // use late binding to retrieve the Subject property of the item
            string subject = "'" +
                itemObj.GetType().InvokeMember("Subject",
                    System.Reflection.BindingFlags.GetProperty,
                    null, itemObj, null).ToString()
                + "'";
            Marshal.ReleaseComObject(itemObj);

            Debug.Print("The InspectorActivate event has occured. The subject is " + subject + ".");

            if (itemEvents.IsConnected)
            {
                Debug.Print("Disconnecting from the previously connected item.");
                itemEvents.RemoveConnection();
            }

            if (GetExplorersCount() > 0) // don't connect if there are NO open explorers 
            {
                // We handle all Outlook item types: MailItem, TaskItem, etc.
                // 'True' means the events class will release the COM object holding a reference to the item shown 
                // in the active inspector when itemEvents.RemoveConnection is called (see the code line above).
                // Note that calling CurrentItem several times creates several COM objects, each of them must be released.
                itemEvents.ConnectTo((inspector as Outlook._Inspector).CurrentItem, true);
                Debug.Print("Connected to this Outlook item.");
            }
            else
                Debug.Print("Do not connect to this Outlook item.");
        }

        private int GetExplorersCount ()
        {
            int count = 0;
            Outlook._Explorers expls = null;

            try
            {
                expls = OutlookApp.Explorers;
                count = expls.Count;
            }
            catch { }
            finally
            {
                if (expls != null) Marshal.ReleaseComObject(expls);
            }
            return count;
        }

        private void adxOutlookEvents_InspectorClose(object sender, object inspector, string folderName)
        {
            Debug.Print("The InspectorClose event has occured.");

            if (itemEvents.IsConnected)
            {
                Debug.Print("Disconnecting from the previously connected item.");
                itemEvents.RemoveConnection();
            }
        }
    }
}

